==> blcr-0.2.0.tar.gz: blcr-0.2.0/examples/pthread_misc/pthread_misc.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: pthread_misc.c,v 1.10 2003/11/14 19:55:20 phargrov Exp $
 */


#include <sys/timeb.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <string.h>

#include "libcr.h"


#define THREADS 2

static int count = 0;
static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

static pthread_t main_tid = (pthread_t)-1;

#define DATELEN 80
static char date[DATELEN];

static char * now(void)
{
    struct timeb when;

    ftime(&when);
    return ctime(&when.time);
}

static int my_callback(void* arg)
{
    int * rc_ptr = (int *)arg;
    int rc;

    if ((int)pthread_self() == main_tid) {
	strncpy(date, now(), DATELEN);
    }

    rc = cr_checkpoint(0);

    if (rc_ptr != NULL) {
	*rc_ptr = rc;
    }

    return 0;
}

static void advance(void)
{
    pthread_mutex_lock(&lock);
    count++;
    pthread_cond_broadcast(&cond);
    pthread_mutex_unlock(&lock);
}

static void wait_for(int want)
{
    pthread_mutex_lock(&lock);
    while (count < want) {
	pthread_cond_wait(&cond, &lock);
    }
    pthread_mutex_unlock(&lock);
}

static void * thread(void * arg)
{
    int tid = (int)pthread_self();
    int rc;

    printf("STARTING thread w/ tid %d\n", tid);
    rc = (int)cr_init();
    if (rc < 0) {
	exit(-1);
    }

    (void)cr_register_callback(my_callback, &rc, CR_SIGNAL_CONTEXT);
    advance();
    wait_for(THREADS + 1);
    if (rc) {
	printf("RESTARTING tid %d, from checkpoint taken %s", tid, date);
    } else {
	printf("CONTINUING tid %d, after taking checkpoint\n", tid);
    }
    printf("EXITING tid %d at %s", tid, now());

    return NULL;
}

int main(void)
{
    pthread_t th[THREADS];
    int rc;
    int i;
	
    printf("Starting main process w/ pid %d\n", (int)getpid());
    main_tid = pthread_self();

    rc = (int)cr_init();
    if (rc < 0) {
	exit(-1);
    }

    (void)cr_register_callback(my_callback, &rc, CR_SIGNAL_CONTEXT);

    // Start one or more threads
    for (i = 0; i < THREADS; ++i) {
	pthread_create(&th[i], NULL, thread, NULL);
    }

    // Wait for them to initialize
    wait_for(THREADS);
    
    cr_request(); // should cause a checkpoint NOW

    if (rc) {
	printf("RESTARTING main thread, from checkpoint taken %s", date);
    } else {
	printf("CONTINUING main thread, after taking checkpoint\n");
    }

    advance();

    for (i = 0; i < THREADS; ++i) {
	rc = pthread_join(th[i], NULL);
	if (rc < 0) {
	    perror("pthread_join()");
	    exit(-1);
	}
    }

    printf("EXITING main thread (pid %d) at %s", (int)getpid(), now());

    return 0;
}

==> blcr-0.6.2.tar.gz: blcr-0.6.2/examples/pthread_misc/pthread_misc.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: pthread_misc.c,v 1.11 2005/07/07 21:21:16 phargrov Exp $
 */


#include <sys/timeb.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <string.h>

#include "libcr.h"


#define THREADS 2

static int count = 0;
static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

static pthread_t main_tid = (pthread_t)-1;

#define DATELEN 80
static char date[DATELEN];

static char * now(void)
{
    struct timeb when;

    ftime(&when);
    return ctime(&when.time);
}

static int my_callback(void* arg)
{
    int * rc_ptr = (int *)arg;
    int rc;

    if (pthread_self() == main_tid) {
	strncpy(date, now(), DATELEN);
    }

    rc = cr_checkpoint(0);

    if (rc_ptr != NULL) {
	*rc_ptr = rc;
    }

    return 0;
}

static void advance(void)
{
    pthread_mutex_lock(&lock);
    count++;
    pthread_cond_broadcast(&cond);
    pthread_mutex_unlock(&lock);
}

static void wait_for(int want)
{
    pthread_mutex_lock(&lock);
    while (count < want) {
	pthread_cond_wait(&cond, &lock);
    }
    pthread_mutex_unlock(&lock);
}

static void * thread(void * arg)
{
    int tid = (int)pthread_self();
    int rc;

    printf("STARTING thread w/ tid %d\n", tid);
    rc = (int)cr_init();
    if (rc < 0) {
	exit(-1);
    }

    (void)cr_register_callback(my_callback, &rc, CR_SIGNAL_CONTEXT);
    advance();
    wait_for(THREADS + 1);
    if (rc) {
	printf("RESTARTING tid %d, from checkpoint taken %s", tid, date);
    } else {
	printf("CONTINUING tid %d, after taking checkpoint\n", tid);
    }
    printf("EXITING tid %d at %s", tid, now());

    return NULL;
}

int main(void)
{
    pthread_t th[THREADS];
    int rc;
    int i;
	
    printf("Starting main process w/ pid %d\n", (int)getpid());
    main_tid = pthread_self();

    rc = (int)cr_init();
    if (rc < 0) {
	exit(-1);
    }

    (void)cr_register_callback(my_callback, &rc, CR_SIGNAL_CONTEXT);

    // Start one or more threads
    for (i = 0; i < THREADS; ++i) {
	pthread_create(&th[i], NULL, thread, NULL);
    }

    // Wait for them to initialize
    wait_for(THREADS);
    
    cr_request(); // should cause a checkpoint NOW

    if (rc) {
	printf("RESTARTING main thread, from checkpoint taken %s", date);
    } else {
	printf("CONTINUING main thread, after taking checkpoint\n");
    }

    advance();

    for (i = 0; i < THREADS; ++i) {
	rc = pthread_join(th[i], NULL);
	if (rc < 0) {
	    perror("pthread_join()");
	    exit(-1);
	}
    }

    printf("EXITING main thread (pid %d) at %s", (int)getpid(), now());

    return 0;
}
